/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.classifiers;

import java.util.Objects;

/**
 *
 * @author Benjamin Paassen - bpaassen@techfak.uni-bielfeld.de
 */
public class Tuple<A, B> {

	private A first;
	private B second;

	public Tuple() {
	}

	public Tuple(A first, B second) {
		this.first = first;
		this.second = second;
	}

	public void setFirst(A first) {
		this.first = first;
	}

	public A getFirst() {
		return first;
	}

	public void setSecond(B second) {
		this.second = second;
	}

	public B getSecond() {
		return second;
	}

	@Override
	public int hashCode() {
		int hash = 7;
		hash = 11 * hash + Objects.hashCode(this.first);
		hash = 11 * hash + Objects.hashCode(this.second);
		return hash;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final Tuple<?, ?> other = (Tuple<?, ?>) obj;
		if (!Objects.equals(this.first, other.first)) {
			return false;
		}
		return true;
	}

	@Override
	public String toString() {
		final StringBuilder out = new StringBuilder();
		out.append("(");
		out.append(first.toString());
		out.append(",");
		out.append(second.toString());
		out.append(")");
		return out.toString();
	}

}
