/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibi.techfak.scie.classifiers;

import de.unibi.techfak.scie.classifiers.data.ClassificationResult;
import de.unibi.techfak.scie.classifiers.data.DataPoint;
import de.unibi.techfak.scie.classifiers.data.FeatureDictionary;
import de.unibi.techfak.scie.classifiers.data.FeatureMap;
import de.unibi.techfak.scie.classifiers.data.LabeledDataPoint;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Random;
import static junit.framework.Assert.assertEquals;
import junit.framework.TestCase;

/**
 *
 * @author Benjamin Paassen - bpaassen@techfak.uni-bielfeld.de
 */
public class ClassifierEvaluationTest extends TestCase {

	public ClassifierEvaluationTest(String testName) {
		super(testName);
	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	public void testEvaluation() {
		final FeatureDictionary dict = new FeatureDictionary();
		dict.addFeature("test");
		//set up a trivial classifier, that sets 0.5 feature value as threshold.
		final Classifier classifier = new TrivialClassifier(dict);
		//set up some data, that have somewhat random labels.
		final ArrayList<LabeledDataPoint> data = new ArrayList<>();
		final Random random = new Random();
		for (double value = -100; value <= 100; value += 1) {
			final boolean label = value > 0;
			final double noisyValue = value + 5 * random.nextGaussian();
			data.add(new LabeledDataPoint(label, new TrivialDataPoint(noisyValue)));
		}
		final ClassifierEvaluation.SetEvaluation result
				= new ClassifierEvaluation.SetEvaluation(classifier, data, 51);
		assertEquals(1, result.getArea_under_roc(), 0.1);
	}

	private static class TrivialClassifier implements Classifier {

		private final FeatureDictionary dict;

		public TrivialClassifier(FeatureDictionary dict) {
			this.dict = dict;
		}

		@Override
		public void train(ArrayList<LabeledDataPoint> trainingData) {
			throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
		}

		@Override
		public ClassificationResult classify(DataPoint data) {
			final FeatureMap features = data.getFeatureRepresentation(dict, false);
			final double value = features.getValue("test");
			final boolean label = value > 0;
			//Use a logistic regression approach to calculate the confidence.
			final double confidence = 1 / (1 + Math.exp(-value));
			return new ClassificationResult(data, label, confidence);
		}

		@Override
		public void writeParamters(Writer parameterOut, Writer featuresOut) throws IOException {
			throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
		}

		@Override
		public void readParameters(Reader parameterIn, Reader featuresIn) throws IOException {
			throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
		}

		@Override
		public FeatureDictionary getFeatureDictionary() {
			return dict;
		}

	}

	private static class TrivialDataPoint implements DataPoint {

		private final double value;

		public TrivialDataPoint(double value) {
			this.value = value;
		}

		@Override
		public FeatureMap getFeatureRepresentation(FeatureDictionary dict,
				boolean addFeaturesToDictionary) {
			final FeatureMap out = new FeatureMap(dict);
			out.addNumericFeature("test", value, addFeaturesToDictionary);
			return out;
		}

	}

}
