/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.classifiers.data;

import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.SortedMap;
import junit.framework.TestCase;

/**
 *
 * @author Benjamin Paassen - bpaassen(at)techfak.uni-bielefeld.de
 */
public class FeatureDictionaryTest extends TestCase {

	public FeatureDictionaryTest(String testName) {
		super(testName);
	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/**
	 * Test of addFeature method, of class FeatureDictionary.
	 */
	public void testAddFeature() {
		final FeatureDictionary instance = new FeatureDictionary();
		instance.addFeature("test");
		final SortedMap<String, Integer> indexMapping = instance.getIndexMapping();
		final SortedMap<Integer, String> nameMapping = instance.getNameMapping();
		assertEquals(1, indexMapping.size());
		assertEquals(1, nameMapping.size());
		assertEquals(new Integer(1), indexMapping.get("test"));
		assertEquals("test", nameMapping.get(1));
	}

	/**
	 * Test of hasFeature method, of class FeatureDictionary.
	 */
	public void testHasFeature() {
		final FeatureDictionary instance = new FeatureDictionary();
		instance.addFeature("test");
		assertTrue(instance.hasFeature("test"));
		assertFalse(instance.hasFeature("test2"));
	}

	/**
	 * Test of getFeatureIndex method, of class FeatureDictionary.
	 */
	public void testGetFeatureIndex() {
		final FeatureDictionary instance = new FeatureDictionary();
		assertFalse(instance.hasFeature("test"));
		instance.addFeature("test");
		assertEquals(1, instance.getFeatureIndex("test"));
		assertTrue(instance.hasFeature("test"));
	}

	/**
	 * Test of getFeatureName method, of class FeatureDictionary.
	 */
	public void testGetFeatureName() {
		final FeatureDictionary instance = new FeatureDictionary();
		instance.addFeature("test");
		assertEquals("test", instance.getFeatureName(1));
	}

	/**
	 * Test of getNumberOfFeatures method, of class FeatureDictionary.
	 */
	public void testGetNumberOfFeatures() {
		final FeatureDictionary instance = new FeatureDictionary();
		assertEquals(0, instance.getNumberOfFeatures());
		instance.addFeature("test");
		assertEquals(1, instance.getNumberOfFeatures());
	}

	/**
	 * Test of store method, of class FeatureDictionary.
	 */
	public void testStore() throws Exception {
		final StringWriter out = new StringWriter();
		final FeatureDictionary instance = new FeatureDictionary();
		instance.addFeature("complexFeatureName:2//*");
		instance.store(out);
		out.close();
		final String storage = out.toString();
		final FeatureDictionary loadedInstance = new FeatureDictionary();
		loadedInstance.load(new StringReader(storage));
		assertEquals(instance, loadedInstance);

	}

}
